#Created by R. David Murray (rdmurray@bitdance.com).
#This code is placed in the public domain.

#Version 0.0.1
__version__ = "$Id: ParmFile.py,v 1.3 2001/12/10 01:09:52 rdmurray Exp $"
#Changes for version 0.0.2:
#  If the default parameter was a ParmFile, the value of a key would be
#  expanded prematurely, and the % escape did not work correctly because
#  the value of the key would be expanded twice.  Fixed _expandkey to
#  check for ParmFile case and use .data attribute to avoid expansion.

"""
  Parse a parameter setting file into a dictionary of values

  A parameter file consists of comment lines, blank lines, and parameter
  lines.  Comment lines have a '#' as the first non-blank character.
  Parameter setting lines are of the form:

    parametername = parametervalue

  parametervalue may contain 'substitution points' of the form:

    %(parametername)

  A parameter name used thus must either be set elsewhere in the
  parameter file, or have been passed in to the parsing function
  in the parameter defaults dictionary.  All substitutions are done
  late, on demand.  (Circular references will result in infinate
  recursion.) To include a litteral %( combination in a paramter
  value, double the %.

  Parameter names are limited to letters, numbers, and the '_'
  character, and are case sensitive.  Leading and trailing blanks
  on parameter values are stripped.

  The module provides a single object and two errors:

    ParmFile(fileobject,defaultdict)
    ParseError
    SubstitutionError

  SubstitutionError is a subclass of KeyError.  The module will
  also return KeyError if a direct lookup (eg: index reference)
  fails.

  All parameter values are strings immediately after parsing, but
  you may update the values as required:  ParmFile supports the
  full UserDict interface.  A ParmFile may be passed as the
  'defaults' parameter of a ParmFile instantiation call.  So may
  the 'environ' dictionary from the the os module.

  The ParmFile object implements the UserDict interface, and adds two
  additional lookup methods:

    isOn(parmname,default=None):boolean
    isOff(parmname,default=None):boolean

  These routines check that the parameter is either not present, or has
  one of the following values:

    yes, y, t, true, on
    no, n, f, false, off

  Case is ignored.  isOn returns true for the first set, isOff returns
  true for the second set.  If default is None, then the parameter
  being missing causes a KeyError.  If a default is specified, it is
  returned in the case where the parameter is not in the ParmFile.

  ParmFile is designed to be used in a cascaded fashion: read a master
  ParmFile to get defaults and pointers to instances, then read an
  instance's ParmFile, passing it the master ParmFile as the default.
  If you prefer to have everything in one file, you can use the following
  method

    setPrefix(id)

  After calling this method, ParmFile lookup methods will first look
  for a the indicated parameter with id prefixed.  Only if that version
  of the parameter name is not found will it look for the unadorned
  parameter name.
"""

from UserDict import UserDict
from xfile import xfile
from string import strip, find, digits, letters, translate, lower

class ParseError(Exception): pass
class SubstitutionError(Exception): pass


class ParmDict(UserDict):
  """Helper class holding the values from a particular input file"""

  def __init__(self,filename,dict=None):
    UserDict.__init__(self,dict)
    self.filename = filename
    self.linenos = {}

  def setLineno(self,key,lineno): self.linenos[key] = lineno
  def getLineno(self,key): return self.linenos.get(key,"<set by program>")
  def getFilename(self,key=None): return self.filename
  def __repr__(self): return "%s(%s,%s)" % (self.__class__.__name__,
    self.filename,repr(self.data))
  def clear(self): UserDict.clear(self); self.filename = None; self.linenos={}


class ParmFile(UserDict):

  _goodparmchars = letters+digits+'_'

  def __init__(self,fileobject,defaultdict={}):
    self.data = ParmDict(getattr(fileobject,'name','<Unknown>'))
    self._defaults = defaultdict
    self._prefix = None
    parmfile = xfile(fileobject,selector=self._notComment,
      parser=self._parseline)
    try: 
      for param, value in parmfile:
        self.data[param] = value
        self.data.setLineno(param,parmfile.lineno())
    except ParseError, msg: raise ParseError("%s on line %s" % (msg,
      parmfile.lineno()))

  def _notComment(self,line):
    line = strip(line[:-1])
    if len(line)==0 or line[0]=='#': return 0
    return 1

  def _parseline(self,line):
    b = find(line,'=')
    if b<0: raise ParseError("missing '=' sign")
    param = strip(line[:max(b,0)])
    if len(param)==0: raise ParseError("Null parameter name")
    if translate(param,'x'*256,self._goodparmchars):
      raise ParseError("Invalid characters in parameter name '%s'" % param)
    return param, strip(line[b+1:])

  def setPrefix(self,prefix):
    self._prefix = prefix

  def getLineno(self,key):
    if self.data.has_key(key): return self.data.getLineno(key)
    if hasattr(self._defaults,'getLineno'): return self._defaults.getLineno(key)
    return '<Unknown>'

  def getFilename(self,key=None):
    if key is None: return self.data.getFilename()
    if self.data.has_key(key): return self.data.getFilename(key)
    if hasattr(self._defaults,'getFilename'):
      return self._defaults.getFilename(key)
    return '<from defaults>'

  def _expandkey(self,source,key):
    #If ParmFile, get from data so value isn't expanded prematurely
    if isinstance(source,ParmFile): value = source.data[key]
    else: value = source[key]
    val = ''
    while 1:
      p = find(value,'%(')
      if p<0: break
      if p>0 and value[p-1]=='%':
        val = val + value[:p]
        value = value[p+1:]
        continue
      val = val + value[:p]
      value = value[p+2:]
      e = find(value,')')
      if e<0:
        self._raiseError(ParseError,"Missing close paren on % code",key,source)
      subname = value[:e]
      value = value[e+1:]
      try: val = val + self.get(subname)
      except SubstitutionError, msg: raise SubstitutionError, msg
      except KeyError:
        self._raiseError(SubstitutionError,"No value known for %s" % \
          subname,key,source)
    return val + value

  def _raiseError(self,exc,msg,key,source):
    if isinstance(source,ParmFile) or isinstance(source,ParmDict):
      raise exc("%s (line %s of %s)" % (msg,source.getLineno(key),
        source.getFilename()))
    else:
      raise exc("%s in default value %s" % (msg,source[key]))

  def __repr__(self): return "%s(%s,%s,%s)" % (self.__class__.__name__,
    self.data.getFilename(),repr(self.data.data),repr(self._defaults))

  def has_key(self,key):
    if self._prefix:
      pkey = self._prefix+key
      if self.data.has_key(pkey) or self._defaults.has_key(pkey): return 1
    if self.data.has_key(key) or self._defaults.has_key(key): return 1
    return 0

  def get(self,key,*args):
    if self._prefix:
      pkey = self._prefix+key
      if self.data.has_key(pkey): return self._expandkey(self.data,pkey)
      if self._defaults.has_key(pkey):
        return self._expandkey(self._defaults,pkey)
    if self.data.has_key(key): return self._expandkey(self.data,key)
    if self._defaults.has_key(key): return self._expandkey(self._defaults,key)
    if len(args)==1: return self._expandkey({key: args[0]},key)
    if len(args)==0: raise KeyError, key
    raise TypeError("function requires at most 2 arguments; %s given" % \
      len(args))

  def __getitem__(self,item): return self.get(item)

  def _isOn(self,parmname,default):
    if not self.has_key(parmname):
      if default is not None: return default
      else: raise KeyError(parmname)
    parm = lower(self[parmname])
    ison = parm in ('yes','y','t','true','on') 
    if not ison and parm not in ('no','n','f','false','off'):
      raise ParseError(("Parameter %s must have true or false value but " +\
        "has value '%s' on line %s of file %s") % (parmname,parm,
        self.getLineno(parmname),self.getFilename(parmname)))
    return ison

  def isOn(self,parmname,default=None):
    return self._isOn(parmname,default)

  def isOff(self,parmname,default=None):
    if default is not None: default = not default
    return not self._isOn(parmname,default)

  def clear(self):
    self.data.clear()
    self._defaults = {}
    self._prefix = None

  def keys(self): return self.data.keys()+self._defaults.keys()
  def items(self): return self.data.items()+self._defaults.items()
  def values(self): return self.data.values()+self._defaults.values()
