#Created by R. David Murray (rdmurray@bitdance.com).
#This code is placed in the public domain.

__version__ = "$Id: ParmFileTests.py,v 1.5 2001/12/10 01:09:52 rdmurray Exp $"

#For version 0.0.1

from unittest import TestCase, makeSuite, TextTestRunner
from ParmFile import ParmFile, ParseError, SubstitutionError

#Note: I'd prefer if this were imported from xfileTests, but their seems
#to be no graceful way to manage that portably.
class FakeFile:

  def __init__(self,l,missingnewline=0):
    self.data = l
    self.pointer = 0
    self.missingnewline = missingnewline
  def readline(self):
    if self.pointer<0 or self.pointer>=len(self.data): return ''
    self.pointer = self.pointer + 1
    if self.missingnewline and self.pointer==len(self.data):
      return self.data[self.pointer-1]
    return self.data[self.pointer-1]+'\n'
  def tell(self): return self.pointer
  def seek(self,i): self.pointer = i

class TestCases(TestCase):

  def setUp(self):
    self.parmfile = ParmFile(FakeFile([
      '#This is a test',
      '',
      'parm1 =    a   ',
      'PaRm2=test',
      'parm3 = %(parm1)/%(PaRm2)',
      'parm4 = %(Default)',
      'parmX = %%(test)',
      'badparmsyn = %(test',
      'badparmkey = %(default)',
      'override_parm1 = c',
      'useset = %(setinprogram)',
    ]),{'Default':'bler  t:%(parm1)'})
    self.parmfile2 = ParmFile(FakeFile([
      '#Test two',
      'default = b',
    ]),self.parmfile)  
    self.nestedparmfile = ParmFile(FakeFile([
      '#This is a test',
      '',
      'parm1 =    a   ',
      'PaRm2=test',
      'parm3 = %(parm1)/%(PaRm2)',
      'parm4 = %(Default)',
      'parmX = %%(test)',
      'badparmsyn = %(test',
      'badparmkey = %(default)',
      'override_parm1 = c',
      'useset = %(setinprogram)',
    ]),ParmFile(FakeFile([
      'Default = bler  t:%(parm1)'
    ])))

  def assertEQ(self,res,correct,msg=''):
    assert res==correct, "'%s' <> '%s' %s" % (repr(res),repr(correct),msg)

  def assertInitRaises(self,errclass,initargs):
    self.assertRaises(errclass,ParmFile,FakeFile(initargs))

  def testParse(self):
    p = self.parmfile
    self.assertEQ(p['parm1'],'a')
    self.assertEQ(p['PaRm2'],'test')
    self.assertEQ(p['parm3'],'a/test')
    self.assertEQ(p['parm4'],'bler  t:a')
    self.assertEQ(p['parmX'],'%(test)')
    self.assertRaises(SubstitutionError,p.get,'badparmkey')
    self.assertRaises(ParseError,p.get,'badparmsyn')

  def testSet(self):
    p = self.parmfile
    self.assertEQ(p['parm1'],'a')
    p['parm1'] = 'b'
    self.assertEQ(p['parm1'],'b')
    self.assertEQ(p['PaRm2'],'test')
    self.assertEQ(p['parm3'],'b/test')
    self.assertEQ(p['Default'],'bler  t:b')
    self.assertEQ(p['parm4'],'bler  t:b')

  def testSetNested(self):
    p = self.nestedparmfile
    self.assertEQ(p['parm1'],'a')
    p['parm1'] = 'b'
    self.assertEQ(p['parm1'],'b')
    self.assertEQ(p['PaRm2'],'test')
    self.assertEQ(p['parm3'],'b/test')
    self.assertEQ(p['Default'],'bler  t:b')
    self.assertEQ(p['parm4'],'bler  t:b')

  def testPrefix(self):
    p = self.parmfile
    self.assertEQ(p['parm1'],'a')
    p.setPrefix('override_')
    self.assertEQ(p['parm1'],'c')
    self.assertEQ(p['PaRm2'],'test')
    self.assertEQ(p['parm3'],'c/test')
    self.assertEQ(p['parm4'],'bler  t:c')

  def testIsOn(self):
    p = self.parmfile
    p['parm5'] = 'on'
    self.assertEQ(p.isOn('parm5'),1)
    self.assertEQ(p.isOn('parm6',0),0)
    self.assertRaises(KeyError,p.isOn,'parm6')
    p['parm6'] = 'blert'
    self.assertRaises(ParseError,p.isOn,'parm6')

  def testIsOff(self):
    p = self.parmfile
    p['parm5'] = 'off'
    self.assertEQ(p.isOff('parm5'),1)
    self.assertEQ(p.isOff('parm6',0),0)
    self.assertRaises(KeyError,p.isOff,'parm6')
    p['parm6'] = 'of'
    self.assertRaises(ParseError,p.isOff,'parm6')

  def testHas_key(self):
    p = self.parmfile
    self.assertEQ(p.has_key('parm1'),1)
    self.assertEQ(p.has_key('blert'),0)

  def testGet(self):
    p = self.parmfile
    self.assertEQ(p.get('parm1'),'a')
    self.assertEQ(p.get('parm1','x'),'a')
    self.assertEQ(p.get('blert','x'),'x')
    self.assertRaises(KeyError,p.get,'blert')

  def testMissingEquals(self):
    self.assertInitRaises(ParseError,['invalid line'])

  def testMissingParmName(self):
    self.assertInitRaises(ParseError,['= invalid line'])

  def testBadChars(self):
    self.assertInitRaises(ParseError,['x89*&6 = invalid line'])

  def testMissingParen(self):
    self.assertInitRaises(ParseError,['%(badline'])

  def testNoValue(self):
    self.assertInitRaises(ParseError,['%(badname)'])

  def testNestedParmFileLateSub(self):
    p = self.parmfile2
    self.assertEQ(p['badparmkey'],'b')

  def testNestedEscape(self):
    p = self.parmfile2
    self.assertEQ(p['parmX'],'%(test)')

  def testSetValue(self):
    p = self.parmfile
    p['Default'] = 'test'
    self.assertEQ(p['parm4'],'test')

  def testSetInProgram(self):
    p = self.parmfile
    p['setinprogram'] = 'test'
    self.assertEQ(p['useset'],'test')
    self.assertEQ(p.has_key('setinprogram'),1)

suite = makeSuite(TestCases,'test')

if __name__ == '__main__':
  from sys import exit
  if not TextTestRunner().run(suite).wasSuccessful(): exit(1)
