"""Tools for parsing command arguments.

The routines in this module handle the basic tasks involved in
parsing a command line.  The module defines one error, ParseError,
that it raises whenever it encounters a parsing error.  The error
value is an error message suitable for end user consumption.  A
routine using this module will normally trap the error and use the
error argument as output to the user.  The routine 'die' can be
used for this purpose when the command source is a command typed
at the OS prompt; it prefixes the output message with the command
name from argv, so the optional name argument to the parsing routines
is not needed in may cases.  The 'args' values are a list, such as
sys.argv.

Most of the routines take the form:

	data, args = routine(args,name)

args is the remaining argument string to be processed.  On the
output side, data is whatever the routine was called to parse out,
and args is the remainder of the argument string with that data
removed.  name is the name of the subcommand whose arguments are
being parsed, and will be included in any error message that might
be generated.  It is generally optional.  If additional arguments
are needed, they will occur between args and name.

A brief example may make this clearer:

args = sys.argv[1:]
try:
	onearg, args = nonnullarg(args)
	nomoreargs(args)
except ParseError, msg: die(msg)

"""

class ParseError(Exception): pass

def listvalues(list,sort=0):
	"""Produces an english phrase representing list.
		res = listvalues(list,sort=0)
	Treats each item in list as an entity, separating them with commas
	and spaces, and the last two (if more than one) by 'and'.  If sort
	is set true, the list is sorted first.
	"""
	l = list[:]
	if sort: l.sort()
	if len(l)==0: return ''
	if len(l)==1: return l[0]
	if len(list)>2: head = reduce(lambda x,y: x + ', ' + str(y),l[:-1])
	else: head = l[0]
	return str(head) + ' and ' + str(l[-1])

def _listvalidvalues(list,sort=1):
	"""Produces an english phrase giving list as a list of valid keywords.
		res = _listvalidvalues(list)
	A listvalues() phrase is prefixed with 'valid values are '.
	Unless the second argument is set false, the list is sorted
	alphabetically.
	"""
	if len(list)==0: return 'actually, there are no valid values'
	if len(list)==1: return 'currently the only valid value is %s' % \
		list[0]
	return 'valid values are ' + listvalues(list,sort)

def cmdname():
	"""Return a simple printable version of the name we were called by.
		name = cmdname()
	sys.argv[0] contains the name the shell called us by, but if a path
	was used that can be ugly.  So this strips out just the command name.
	"""
	from sys import argv
	from string import split
	return split(argv[0],'/')[-1]

def die(msg,rc=1):
	"""Write an error message to the console and exit with a non-zero rc.
		die(msg[,rc])
	cmdname is prefixed to the message.  '1' is the default
	return code.  If msg is blank, the routine is equivalent
	to sys.exit(rc).
	"""
	from sys import stderr, exit
	if msg=='': exit(rc)
	stderr.write("%s: %s\n" % (cmdname(),msg))
	exit(rc)

def getoptions(args,optlist,longoptlist=[]):
	"""Parse traditional Unix options into a dictionary of values
		getoptions(args,optlist):optdict,args
	Uses the getopt module, which mimics the Unix getopt.  See the
	getopt man page for an explanation of optlist/longoptlist.  Result
        is a dictionary keyed by the option letter/name.  If an option
        is specified more than once in an input string, only the last
        value is kept.
        """
	from getopt import getopt, error
	from string import strip
	try: opts, args = getopt(args,optlist,longoptlist)
	except error, msg: raise ParseError,msg
	options = {}
        for opt,val in opts:
          if opt[0]=='-': opt = opt[1:]
          if opt[0]=='-': opt = opt[1:]
          options.update({opt:val})
	return options, args

def nomoreargs(args,name=''):
	"""Make sure we have exhausted the argument string.
		nomoreargs(args[,name])
	Too many arguments can be just as bad as too few...
	"""
	from string import join
	if len(args)==0: return
	if name<>'': name = ' after ' + name
	raise ParseError, ("extra text '%s' found" + name) % join(args)

def nonnullarg(args,desc='',name=''):
	"""Make sure there is a next token, and return it.
		token, args = nonnullarg(args[,name])
	Two possible messages: 'missing argument', or 'null argument'.
	If 'desc' is supplied, the first message is "missing %s", where %s
	is desc.  If the next token *does* exist it is returned unaltered.
	"""
	from string import strip
	if name<>'': name = ' for ' + name
	if desc=='': desc = "argument"
	if len(args)<1: raise ParseError, 'missing ' + desc + name
	if len(strip(args[0]))==0: raise ParseError, 'null ' + desc + name
	return (args[0], args[1:])

def getint(args,desc='',name=''):
	"""Make sure the next token is an integer and return it.
		i, args = getint(args[,desc[,name]])
	Possible messages are missing arg and expected integer but found X.
	"""
	from string import atoi
	if desc=='': desc = 'integer'
	if name<>'': name = ' for ' + name
	i, args = nonnullarg(args,desc)
	try: res = atoi(i)
	except ValueError: raise ParseError, 'Expected %s but' + \
		'found "%s"' % (desc,i)
	return (res, args)

def oneof(args,list,name=''):
	"""Make sure next token is in list.
		match, args = oneof(args,list[,name])
	Possible error messages are missing arg and invalid keyword.
	In the latter case the list of valid values is given.  Case is
	respected.  If the keyword is found, it is returned.
	"""
	key, args = nonnullarg(args,'keyword (' + _listvalidvalues(list) + \
		')',name)
	if key in list: return (key, args)
	if name<>'': name = ' for ' + name
	raise ParseError, "invalid keyword '" \
		+ key + "'" + name + ' (' + _listvalidvalues(list) + ')'

def matchoneof(args,list,name=''):
	"""Unambiguous prefix match of next token against list.
		match, args = matchoneof(args,list[,name])
	Possible error messages are missing arg, invalid keyword, and
	ambiguous match.  In the latter two cases the list of valid values
	is given.  Case is ignored.  If an unambiguous match is found,
	the full length word from list is returned as 'match'.
	"""
	from regex import compile, casefold
	key, args = nonnullarg(args,'keyword (' + _listvalidvalues(list) + \
		')',name)
	if name<>'': name = ' for ' + name
	keymatch = compile(key,casefold)
	count = len(key)
	matches = []
	for k in list:
		if keymatch.match(k)==count: matches.append(k)
	if len(matches)==0: raise ParseError, "invalid keyword '" \
		+ key + "'" + name + ' (' + _listvalidvalues(list) + ')'
	if len(matches)>1: raise ParseError, "ambiguous keyword '" \
		+ key + "'" + name + ' (' + _listvalidvalues(list) + ')'
	return (matches[0],args)

def getexistingfilename(args,defdir='',name=''):
	"""Parse next token as the filename of an existing file.
		name, args = getexistingfilename(args[,defdir[,name]])
	Make sure next token is the name of an existing file, and returns
	the filename string.  If defdir is given and the token is not an
	absolute path name, then if the file does not exist (the token
	was not valid relative to the current directory) a second try is
	made with defdir prepended.  The name returned is the full pathname
        of the file if it was not found in the current directory.
	"""
	import os
	filename, args = nonnullarg(args,'filename',name)
	if name<>'': name = ' for ' + name
	if os.path.isfile(filename): return filename, args
	if not defdir or os.path.isabs(filename):
		raise ParseError, 'Cannot file file ' + filename + name
	absfilename = os.path.join(defdir,filename)
	if os.path.isfile(absfilename): return absfilename, args
	raise ParseError, 'Cannot find file ' + filename + ' or ' + \
		absfilename + name

def getfileforread(args,defdir,name=''):
	"""Parse next token as existing file to open for reading.
		handle, args = getfileforread(args[,defdir[,name]])
	Make sure next token is the name of an existing file, and returns
	an open file handle on the file.  If defdir is given and the token
	does not start with '/', then if the file does not exist (the token
	was not valid relative to the current directory) a second try is
	made with defdir prepended.
	"""
	filename, args = getexistingfilename(args,defdir,name)
	return open(filename,'r'), args
